#include <malloc.h>
#include <string.h>
#include <stdio.h>
#include "bmp.h"
#include "file.h"
#include "endian.h"
#include "hash.h"

#define	ID_BM	0x4D42			/*  */

uint16_t ReadU16LE(FILE** file) {
	uint16_t res;
	fread(&res, sizeof(res), 1, *file);
	#ifdef IS_BIGENDIAN
	res = res<<8 | (res>>8 & 0xFF);
	#endif
	return res;
}

uint32_t ReadU32LE(FILE** file) {
	uint32_t res;
	fread(&res, sizeof(res), 1, *file);
	#ifdef IS_BIGENDIAN
	res =  res<<24 | (res<<8 & 0x00FF0000) | (res>>8 & 0xFF00) | (res>>24 & 0xFF); 
	#endif
	return res;
}

bool bmpConstruct(BMP* bmp, const char* filepath) {
	memset(bmp, 0, sizeof(BMP));
	FILE* imgfile = fopen(filepath, "rb");
	if(NULL == imgfile) {
		return false;
	}

	fseek(imgfile, 0, SEEK_END);
	size_t size = ftell(imgfile);
	fseek(imgfile, 0, SEEK_SET);

	bmp->fileh.type = ReadU16LE(&imgfile);
	bmp->fileh.size = ReadU32LE(&imgfile);
	bmp->fileh.rsrvd1 = ReadU16LE(&imgfile);
	bmp->fileh.rsrvd2 = ReadU16LE(&imgfile);
	bmp->fileh.ofs = ReadU32LE(&imgfile);

	bmp->infoh.size = ReadU32LE(&imgfile);
	bmp->infoh.width = ReadU32LE(&imgfile);
	bmp->infoh.height = ReadU32LE(&imgfile);
	bmp->infoh.planes = ReadU16LE(&imgfile);
	bmp->infoh.bitcnt = ReadU16LE(&imgfile);
	bmp->infoh.compression = ReadU32LE(&imgfile);
	bmp->infoh.imgsize = ReadU32LE(&imgfile);
	bmp->infoh.xppm = ReadU32LE(&imgfile);
	bmp->infoh.yppm = ReadU32LE(&imgfile);
	bmp->infoh.clrused = ReadU32LE(&imgfile);
	bmp->infoh.clrimportant = ReadU32LE(&imgfile);

	bmp->pal = NULL;

	if(24 != bmp->infoh.bitcnt) {
		fclose(imgfile);
		return false;
	}

	size_t bufsize = size-ftell(imgfile);
	bmp->pixels = malloc(bufsize);
	fread(bmp->pixels, sizeof(char), bufsize, imgfile);

	fclose(imgfile);
	return true;
}

void bmpDestroy(BMP* bmp) {
	free(bmp->pal);
	free(bmp->pixels);
}

void* bmpAbandonPixels(BMP* bmp) {
	void* ret = bmp->pixels;
	bmp->pixels = NULL;
	return ret;
}

uint32_t bmpGetPixel(const BMP* bmp, uint32_t i) {
	switch(bmp->infoh.bitcnt) {
		case 24: {
			uint8_t r = ((uint8_t*)bmp->pixels)[3*i+2];
			uint8_t g = ((uint8_t*)bmp->pixels)[3*i+1];
			uint8_t b = ((uint8_t*)bmp->pixels)[3*i+0];
			return r<<16 | g<<8 | b;
		}
		break;
		default:
			return 0;
	}
}

uint32_t bmpGetPixelCount(const BMP* bmp) {
	return bmp->infoh.width*bmp->infoh.height;
}

uint32_t bmpCountColors(const BMP* bmp) {
	switch(bmp->infoh.bitcnt) {
		case 24: {
			Hash histogram;
			hashConstruct(&histogram);
			for(uint32_t i=0, n=bmpGetPixelCount(bmp); i<n; i++) {
				uint32_t color = bmpGetPixel(bmp, i);
				hashInsert(&histogram, color, color);
			}
			uint32_t ret = hashGetNumKeys(&histogram);
			hashDestroy(&histogram);
			return ret;
	 	}
		break;
		default:
			return 0;
	}
}

