#ifndef BMP_H
#define BMP_H

#include <stdint.h>
#include <stdbool.h>

typedef struct {
	uint16_t	type;
	uint32_t	size;
	uint16_t	rsrvd1;
	uint16_t	rsrvd2;
	uint32_t	ofs;
} BMPFileHeader;

typedef struct {
	uint32_t	size;
	uint32_t	width;
	uint32_t	height;
	uint16_t	planes;
	uint16_t	bitcnt;
	uint32_t	compression;
	uint32_t	imgsize;
	uint32_t	xppm;
	uint32_t	yppm;
	uint32_t	clrused;
	uint32_t	clrimportant;
} BMPInfoHeader;

typedef struct {
	uint8_t	blue;
	uint8_t	green;
	uint8_t	red;
	uint8_t	pad;
} BMPRGBQuad;

typedef struct {
	BMPFileHeader	fileh;
	BMPInfoHeader	infoh;
	BMPRGBQuad*	pal;
	void*		pixels;
} BMP;

/* interface */
bool bmpConstruct(BMP* bmp, const char* filepath);
void bmpDestroy(BMP* bmp);
void* bmpAbandonPixels(BMP* bmp);
uint32_t bmpGetPixel(const BMP* bmp, uint32_t i);
uint32_t bmpGetPixelCount(const BMP* bmp);
uint32_t bmpCountColors(const BMP* bmp);

#endif	/* BMP_H */

