#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include "image.h"
#include "bmp.h"
#include "hash.h"
#include "file.h"
 
Image* imageNew(const char* file) {
	Image* image = malloc(sizeof(Image));
	if(NULL == image)
		abort();
	if(imageConstruct(image, file))
		return image;
	return NULL;
}

void imageDelete(Image* image) {
	imageDestroy(image);
	free(image);
}

bool imageConstruct(Image* image, const char* filepath) {
	memset(image, 0, sizeof(Image));
	const char* ext = fileExtractAfterLast(filepath, '.');
	if(0 == strcmp(ext, "bmp")) {
		BMP bmp;
		if(false == bmpConstruct(&bmp, filepath))
			return false;

		image->width = bmp.infoh.width;
		image->height = bmp.infoh.height;
		uint32_t npix = image->width*image->height;
		//uint32_t ncol = bmpCountColors(&bmp);
		//printf ( "%s has %d unique colors\n", filepath, ncol );
 
		if(24 == bmp.infoh.bitcnt) {
			image->format = IMAGE_FMT_RGBA32;
			image->pixels = malloc(sizeof(uint32_t)*npix);
			for(uint32_t i=0; i<npix; i++) {
				((uint32_t*)image->pixels)[i] = bmpGetPixel(&bmp, i) << 8;
			}
		} else {
			bmpDestroy(&bmp);
			return false;
		}
  		bmpDestroy(&bmp);
	}
	else
		return false;

	imageConvert(image, IMAGE_FMT_RGBA16);
	return true;
}

void imageDestroy(Image* image) {
	free(image->pixels);
	free(image->palette);
}

inline uint32_t imageGetPixelCount(const Image* image) {
	return image->width*image->height;
}

uint32_t imageGetPixel(const Image* image, uint32_t i) {
	switch(image->format) {
		case IMAGE_FMT_RGBA32: {
			return ((uint32_t*)image->pixels)[i];
			break;
		}
		default:
			return 0;
	}

	return 0;
}

void imageConvert(Image* image, ImageFormat format) {
	uint32_t npix = imageGetPixelCount(image);
	if(IMAGE_FMT_RGBA16 == format) {
		uint16_t* newpixels = malloc(npix<<1);
		if(NULL == newpixels)
			abort();

		for(uint32_t i=0; i<npix; i++) {
			uint32_t color = imageGetPixel(image, i);
			newpixels[i] = (((color>>27)&0x1F) << 11) | (((color>>19)&0x1F) << 6) | (((color>>11)&0x1F) << 1) | ((color>>7)&0x1);
		}
		free(image->pixels);
		image->pixels = newpixels;

		image->format = format;
	}

	return;
	/*  */
	Hash hash;
	hashConstruct(&hash);
	for(uint32_t i=0; i<npix; i++) {
		uint32_t color = imageGetPixel(image, i);
		hashInsert(&hash, color, i);
	}
	uint32_t ncol = hashGetNumKeys(&hash);
	if(ncol <= 16 && IMAGE_FMT_CI4 == format) {
		free(image->pixels);
		image->pixels = malloc(npix/2);
		if(NULL == image->pixels)
			abort();

		image->format = format;
		for(uint32_t i=0; i<ncol; i++) {
			const MyPair* pair = hashGetPair(&hash, i);
			for(uint32_t j=0; j<vecGetSize(uint32_t, &pair->vals); j++) {
				uint8_t pixidx = *vecGetConstElement(uint32_t, &pair->vals, j);
				uint8_t* pix = ((uint8_t*)image->pixels + (pixidx>>1));
				if(pixidx & 0x1)
					*pix = (*pix & 0xF0) | pair->key;
				else
					*pix = (*pix & 0x0F) | pair->key << 4;
			}
		}
	}
	hashDestroy(&hash);
}
