#ifndef LAYER_H
#define LAYER_H

#include <stdint.h>
#include "vector.h"
#include "vec_face.h"
#include "vec_vertex.h"
#include "vec_surface.h"
#include "vec_pair.h"
#include "vec_pchar.h"
#include "hash.h"

#define	LAYER_NOT_SET	0xFFFF			/*  */

typedef struct {
	Vec(Vertex)	vts;
	Vec(Face)	pols;
	Vec(Pair)	ptpairs;
	Vector3f	pivot;
	char*		name;
	uint16_t	parent;
	uint16_t	number;
	uint16_t	flags;
} Layer;

typedef Layer* PLayer;

/* interface */
Layer* layerNew(uint16_t number, uint16_t flags, Vector3f* pivot, char* name, uint16_t parent);
void layerDelete(Layer* layer);
void layerConstruct(Layer* layer, uint16_t number, uint16_t flags, Vector3f* pivot, char* name, uint16_t parent);
void layerDestroy(Layer* layer);

uint16_t layerGetNumber(const Layer* layer);
uint32_t layerGetVertexCount(const Layer* layer);
uint32_t layerGetFaceCount(const Layer* layer);
uint32_t layerGetPairCount(const Layer* layer);

Vertex* layerGetVertex(Layer* layer, uint32_t i);
Face* layerGetFace(Layer* layer, uint32_t i);
Pair* layerGetPair(Layer* layer, uint32_t i);
const Vertex* layerGetConstVertex(const Layer* layer, uint32_t i);
const Face* layerGetConstFace(const Layer* layer, uint32_t i);
const Pair* layerGetConstPair(const Layer* layer, uint32_t i);

void layerAddVertex(Layer* layer, Vertex* vertex);
void layerAddFace(Layer* layer, Face* face);
void layerAddPair(Layer* layer, Pair* pair);

void layerCalculateNormals(Layer* layer);
Hash* layerSpecifyGroups(const Layer* layer);

#endif	/* LAYER_H */

