#include "layer_.h"

void layer_Construct(Layer_* layer, uint32_t vts_el_size, uint32_t faces_el_size) {
	vec_Construct(&layer->vts, vts_el_size);
	vec_Construct(&layer->faces, faces_el_size);
}

void layer_Destroy(Layer_* layer) {
	vec_Destroy(&layer->faces);
	vec_Destroy(&layer->vts);
}

inline uint32_t layer_GetVertexCount(const Layer_* layer) {
	return vec_GetSize(&layer->vts);
}

inline uint32_t layer_GetFaceCount(const Layer_* layer) {
	return vec_GetSize(&layer->faces);
}

inline void* layer_GetVertex(Layer_* layer, uint32_t i) {
	return vec_GetElement(&layer->vts, i);
}

inline void* layer_GetFace(Layer_* layer, uint32_t i) {
	return vec_GetElement(&layer->faces, i);
}

inline const void* layer_GetConstVertex(const Layer_* layer, uint32_t i) {
	return vec_GetConstElement(&layer->vts, i);
}

inline const void* layer_GetConstFace(const Layer_* layer, uint32_t i) {
	return vec_GetConstElement(&layer->faces, i);
}

inline void layer_AddVertex(Layer_* layer, const void* vertex) {
	vec_PushBack(&layer->vts, vertex);
}

inline void layer_AddFace(Layer_* layer, const void* face) {
	vec_PushBack(&layer->faces, face);
}

/* void layerCalculateNormals(void* layer) {
	Hash hash;
	hashConstruct(&hash);

	for(uint32_t i=0, n=layerGetFaceCount(layer); i<n;  i++) {
		const Face* face = layerGetConstFace(layer, i);
		for(uint16_t j=0, m=faceGetVertexCount(face); j<m; j++) {
			hashInsert(&hash, faceGetIndex(face, j), i);
		}
	}

	for(uint32_t i=0; i<hashGetNumKeys(&hash); i++) {
		MyPair* pair = hashGetPair(&hash, i);
		Vector3f vnormal = { 0, 0, 0 };
		for(uint32_t j=0, n=vecGetSize(uint32_t, &pair->vals); j<n; j++) {
			const Face* face = layerGetConstFace(layer, *vecGetElement(uint32_t, &pair->vals, j));
			const Vertex* p0 = layerGetConstVertex(layer, faceGetIndex(face, 0));
			const Vertex* p1 = layerGetConstVertex(layer, faceGetIndex(face, 1));
			const Vertex* p2 = layerGetConstVertex(layer, faceGetIndex(face, 2));

			Vector3f u,v;
			vector3fSub(&u, vertexGetConstPosition(p1), vertexGetConstPosition(p0));
			vector3fSub(&v, vertexGetConstPosition(p2), vertexGetConstPosition(p0));
			Vector3f fnormal;
			mathCrossProduct(&fnormal, &u, &v);
			vector3fAdd(&vnormal, &vnormal, &fnormal);
		}
		mathNormalizeVector(&vnormal, &vnormal);

		Vertex* vtx = layerGetVertex(layer, pair->key);
		*vertexGetNormal(vtx) = vnormal;
	}
	hashDestroy(&hash);
} */

