#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include "lwoparser.h"
#include "chunkhandlers.h"

/* chunks */
#define	ID_FORM		'FORM'
#define	ID_TAGS		'TAGS'			/*  */
#define	ID_PNTS		'PNTS'			/*  */
#define	ID_POLS		'POLS'			/*  */
#define	ID_PTAG		'PTAG'			/*  */
#define	ID_SURF		'SURF'			/*  */
#define	ID_LAYR		'LAYR'			/*  */
#define	ID_VMAP		'VMAP'			/*  */
#define	ID_VMAD		'VMAD'			/*  */
#define	ID_CLIP		'CLIP'			/*  */
/* subchunks */
#define	ID_COLR		'COLR'			/*  */
#define	ID_BLOK		'BLOK'			/*  */
#define	ID_IMAP		'IMAP'			/*  */
#define	ID_TMAP		'TMAP'			/*  */
#define	ID_SIZE		'SIZE'			/*  */
#define	ID_IMAG		'IMAG'			/*  */
#define	ID_STIL		'STIL'			/*  */

void lwoparserConstruct(LwoParser* parser) {
	meshConstruct(&parser->mesh);
}

void lwoparserDestroy(LwoParser* parser) {
	meshDestroy(&parser->mesh);
}

bool ReadChunks(ChunkContext* ctx) {

	while(ctx->bytes) {
		/* read header */
		ReadU32(&ctx->id, ctx);
		if(!ctx->issubchunk) {
			ReadU32(&ctx->size, ctx);
			*ctx->parentbytes -= 2*sizeof(uint32_t);
		}
		else {
			uint16_t size16;
			ReadU16(&size16, ctx);
			ctx->size = size16;
			*ctx->parentbytes -= sizeof(ctx->id) + sizeof(size16);
		}

		ctx->bytesafter = ctx->bytes - ctx->size;	/* bytes num after reading chunk */
		if(false == ctx->issubchunk) {
			switch(ctx->id) {
				case ID_FORM:
					HandleFORM(ctx);
					break;
				case ID_SURF: 
					HandleSURF(ctx);
					break;
				case ID_TAGS: 
					HandleTAGS(ctx);
					break;
				case ID_PNTS: 
					HandlePNTS(ctx);
					break;
				case ID_POLS: 
					HandlePOLS(ctx);
					break;
				case ID_PTAG: 
					HandlePTAG(ctx);
					break;
				case ID_LAYR: 
					HandleLAYR(ctx);
					break;
				case ID_VMAP: 
					HandleVMAP(ctx);
					break;
				case ID_VMAD:
					HandleVMAD(ctx);
					break;
				case ID_CLIP:
					HandleCLIP(ctx);
					break;
				default:
					HandleDefault(ctx);
			}
		} else {
			switch(ctx->id) {
				case ID_STIL:
					HandleSTIL_sub(ctx);
					break;
				case ID_IMAG:
					HandleIMAG_sub(ctx);
					break;
				case ID_SIZE:
					HandleSIZE_sub(ctx);
					break;
				case ID_TMAP:
					HandleTMAP_sub(ctx);
					break;
				case ID_BLOK:
					HandleBLOK_sub(ctx);
					break;
				case ID_IMAP:
					HandleIMAP_sub(ctx);
					break;
				case ID_COLR:
					HandleCOLR_sub(ctx);
					break;
				default:
					HandleDefault(ctx);
			}
		}

		/* we have read data  */
		*ctx->parentbytes -= ctx->size;
	}

	/* omit padding byte */
	if((uintptr_t)*ctx->inbuf & 0x1) {
		(*(uint8_t**)ctx->inbuf)++;
		(*ctx->parentbytes)--;
	}

	return true;
}

bool ParseLWO(LwoParser* parser, const uint32_t* inbuf, uint32_t filesize) {
	ChunkContext context = {
		.parser = parser,
		.inbuf = (const void**)&inbuf,
		.bytes = filesize,
		.parentbytes = &filesize,
		.issubchunk = false,
		.data = NULL,
		.layer_number = LAYER_NOT_SET };

	if(false == ReadChunks(&context))
		return false;
	for(uint16_t i=0, n=meshGetLayerCount(&parser->mesh); i<n; i++) {
		Layer* layer = meshGetLayer(&parser->mesh, i);
		layerCalculateNormals(layer);
	}
	return true;
}

