#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lwoparser.h"
#include "converter.h"
#include "vec_pvoid.h"

Vec(Pvoid) g_imagevec;

int main(int argc, char* argv[]) {
	if(1 == argc) {
		printf ( "LWO2ULTRA - LWO2 object to N64 display list converter by smiglo\nUsage: %s [-o outputfile] [-s scalefactor] inputfile\n", argv[0] );
		exit(EXIT_FAILURE);
	}

	/* parse options */
	int opt;
	char* ofilename = NULL;
	double scale = 1.0;
	while( -1 != (opt = getopt(argc, argv, "o:s:"))) {
		switch(opt) {
			case 'o':
				ofilename = (char*)malloc(strlen(optarg) + 1);	
				strcpy(ofilename, optarg);
				break;
			case 's':
				if(0 == (scale = strtod(optarg, NULL))) {
					fprintf(stderr, "Bad numeric value.\n");
					exit(EXIT_FAILURE);
				}
				break;
			default:
				fprintf(stderr, "Usage: %s [-o outputfile] [-s scalefactor] inputfile\n", argv[0]);
				exit(EXIT_FAILURE);
				break;
		}
	}

	if(optind != argc-1) {
		fprintf(stderr, "Usage: %s [-o outputfile] [-s scalefactor] inputfile\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	if(NULL == ofilename) {
		/* default outputfile name */
		ofilename = (char*)malloc(strlen(argv[optind]) + 2 + 1);	/* alloc for string + .extension */
		strcpy(ofilename, argv[optind]);
		strcat(ofilename, ".c");
	}

	/* open input file */
	FILE* infile = fopen(argv[optind], "rb");
	if(NULL == infile) {
		printf("error while opening input file\n");
		exit(EXIT_FAILURE);
	}
	
	/* create output file */
	FILE* outfile = fopen(ofilename, "wb");
	if(NULL == outfile) {
		printf("error while opening output file.\n");
		exit(EXIT_FAILURE);
	}
	fseek(infile, 0, SEEK_END);

	/* get input file size to alloc a buffer */
	size_t insize = ftell(infile);
	if(0 == insize) {
		printf ( "Empty input file.\n" );
		exit(EXIT_FAILURE);
	}
	fseek(infile, 0, SEEK_SET);

	void* inbuf = malloc(insize);
	if(NULL == inbuf) {
		printf("malloc failed\n");
		exit(EXIT_FAILURE);
	}
	size_t n = fread(inbuf, sizeof(char), insize, infile);
	if(n < insize) {
		printf ( "fread failed");
		exit(EXIT_FAILURE);
	}

	/* parse file */
	LwoParser lwoParser;
	lwoparserConstruct(&lwoParser);
	lwoParser.mesh.filename = argv[optind];
	if(!ParseLWO(&lwoParser, inbuf, insize)) {
		printf ( "Parsing failed.\n" );
		exit(EXIT_FAILURE);
	}

	meshScale(&lwoParser.mesh, scale);

	/* convert to display list */
	vecConstruct(Pvoid, &g_imagevec);
	UltraMesh* umesh = converterProcess(&lwoParser.mesh);
	char* dl = ultrameshGenerateDL(umesh);
	vecDestroy(Pvoid, &g_imagevec);
	if(NULL == dl)
		exit(EXIT_FAILURE);

	/* write to output file */
	const char* head = "#include \"myheader.h\"\n\n";
	size_t head_len = strlen(head);
	size_t dl_len = strlen(dl);
	if(fwrite(head, sizeof(char), head_len, outfile) < head_len
	|| fwrite(dl, sizeof(char), dl_len, outfile) < dl_len) {
		printf ( "fwrite failed.\n" );
		exit(EXIT_FAILURE);
	}

	/* close files */
	if(fclose(outfile) || fclose(infile)) {
		printf ( "fclose failed.\n" );
		exit(EXIT_FAILURE);
	}
	free(ofilename);

	lwoparserDestroy(&lwoParser);

	return 0;
}

