#ifndef MESH_H
#define MESH_H

#define	FACE_MAX_VTS	4			/*  */

#include <stdio.h>
#include <stdint.h>
#include "vec_pchar.h"
#include "vec_psurface.h"
#include "vec_player.h"
#include "vec_pclip.h"

typedef struct {
	Vec(Pchar)	tags;
	Vec(PLayer)	layers;
	Vec(PSurface)	surfs;
	Vec(PClip)	clips;
	const char*	filename;
} Mesh;

/* interface */
void meshConstruct(Mesh* mesh);
void meshDestroy(Mesh* mesh);

void meshScale(Mesh* mesh, double scale);

uint16_t meshGetLayerCount(const Mesh* mesh);
uint32_t meshGetVertexCount(const Mesh* mesh);
uint32_t meshGetFaceCount(const Mesh* mesh);
uint32_t meshGetSurfaceCount(const Mesh* mesh);
uint32_t meshGetClipCount(const Mesh* mesh);

Layer* meshGetLayer(Mesh* mesh, uint32_t i);
Layer* meshGetLayerByNumber(Mesh* mesh, uint16_t number);
Surface* meshGetSurface(Mesh* mesh, uint32_t i);
Clip* meshGetClip(Mesh* mesh, uint32_t i);
const Layer* meshGetConstLayer(const Mesh* mesh, uint32_t i);
const Surface* meshGetConstSurface(const Mesh* mesh, uint32_t i);
const Surface* meshGetConstSurfaceByName(const Mesh* mesh, const char* name);
const Clip* meshGetConstClip(const Mesh* mesh, uint32_t i);
const Clip* meshGetConstClipById(const Mesh* mesh, uint32_t id);
const char* meshGetConstTag(const Mesh* mesh, uint32_t i);

/* mesh owns layer, surface and clip */
void meshAddLayer(Mesh* mesh, Layer* layer);
void meshAddSurface(Mesh* mesh, Surface* surface);
void meshAddClip(Mesh* mesh, Clip* clip);

#endif	/* MESH_H */
