#include <stdlib.h>
#include "surface.h"

Surface* surfaceNew(char* name, char* source, const Vector3f* color) {
	Surface* surface = malloc(sizeof(Surface));
	if(NULL == surface)
		abort();
	surfaceConstruct(surface, name, source, color);
	return surface;
}

inline void surfaceDelete(Surface* surface) {
	surfaceDestroy(surface);
	free(surface);
}

void surfaceConstruct(Surface* surface, char* name, char* source, const Vector3f* color) {
	surfaceSetName(surface, name);
	surfaceSetSource(surface, source);
	if(color)
		surfaceSetColor(surface, color);
	vecConstruct(PBlock, &surface->blocks);
}

void surfaceDestroy(Surface* surface) {
	for(uint32_t i=0, n=vecGetSize(PBlock, &surface->blocks); i<n; i++) {
		blockDelete(*vecGetElement(PBlock, &surface->blocks, i));
	}
	free(surface->source);
	free(surface->name);
}

inline void surfaceSetName(Surface* surface, char* name) {
	surface->name = name;
}

inline void surfaceSetSource(Surface* surface, char* source) {
	surface->source = source;
}

inline void surfaceSetColor(Surface* surface, const Vector3f* color) {
	surface->color = *color;
}

uint32_t surfaceGetBlockCount(const Surface* surface) {
	return vecGetSize(PBlock, &surface->blocks);
}

inline const char* surfaceGetName(const Surface* surface) {
	return surface->name;
}

inline Vector3f* surfaceGetColor(Surface* surface) {
	return &surface->color;
}

inline const Vector3f* surfaceGetConstColor(const Surface* surface) {
	return &surface->color;
}

inline const Block* surfaceGetConstBlock(const Surface* surface, uint32_t i) {
	return *vecGetConstElement(PBlock, &surface->blocks, i);
}

void surfaceAddBlock(Surface* surface, Block* block) {
	vecPushBack(PBlock, &surface->blocks, block);
}

