#include <malloc.h>
#include <stdlib.h>
#include "ultralayer.h"

UltraLayer* ultralayerNew(uint16_t matid) {
	UltraLayer* ulayer = malloc(sizeof(UltraLayer));
	if(NULL == ulayer)
		abort();
	ultralayerConstruct(ulayer, matid);
	return ulayer;
}

void ultralayerDelete(UltraLayer* ulayer) {
	ultralayerDestroy(ulayer);
	free(ulayer);
}

inline void ultralayerConstruct(UltraLayer* ulayer, uint16_t matid) {
	layer_Construct(&ulayer->layer_, sizeof(UltraVertex), sizeof(UltraFace));
	ulayer->matid = matid;
}

inline void ultralayerDestroy(UltraLayer* ulayer) {
	layer_Destroy(&ulayer->layer_);
}

inline uint32_t ultralayerGetVertexCount(const UltraLayer* ulayer) {
	return layer_GetVertexCount(&ulayer->layer_);
}

inline uint32_t ultralayerGetFaceCount(const UltraLayer* ulayer) {
	return layer_GetFaceCount(&ulayer->layer_);
}

inline UltraVertex* ultralayerGetVertex(UltraLayer* ulayer, uint32_t i) {
	return layer_GetVertex(&ulayer->layer_, i);
}

inline UltraFace* ultralayerGetFace(UltraLayer* ulayer, uint32_t i) {
	return layer_GetFace(&ulayer->layer_, i);
}

inline uint16_t ultralayerGetMaterialId(const UltraLayer* ulayer) {
	return ulayer->matid;
}

inline const UltraVertex* ultralayerGetConstVertex(const UltraLayer* ulayer, uint32_t i) {
	return layer_GetConstVertex(&ulayer->layer_, i);
}

inline const UltraFace* ultralayerGetConstFace(const UltraLayer* ulayer, uint32_t i) {
	return layer_GetConstFace(&ulayer->layer_, i);
}

inline void ultralayerAddVertex(UltraLayer* ulayer, const UltraVertex* vertex) {
	layer_AddVertex(&ulayer->layer_, vertex);
}

inline void ultralayerAddFace(UltraLayer* ulayer, const UltraFace* face) {
	layer_AddFace(&ulayer->layer_, face);
}

