#include <stdint.h>

#define VEC__STRUCTNAME(T)	Vec##T
#define VEC__F(T, name)		Vec##T##name

/* structures */
#define VEC__DECL(T)			\
typedef struct VEC__STRUCTNAME(T) {	\
	T*		data;		\
	uint32_t	num_el;		\
	uint32_t	max_el;		\
} VEC__STRUCTNAME(T)

/* declaration */
#define Vec(T) 		VEC__STRUCTNAME(T)

/* build function names */
#define vecNew(T)			VEC__F(T, New)()
#define vecDelete(T, pvec)		VEC__F(T, Delete)(pvec)
#define vecConstruct(T, pvec)		VEC__F(T, Construct)(pvec)
#define vecDestroy(T, pvec)		VEC__F(T, Destroy)(pvec)
#define vecResize(T, pvec)		VEC__F(T, Resize)(pvec)
#define vecPushBack(T, pvec, el)	VEC__F(T, PushBack)(pvec, el)
#define vecGetElement(T, pvec, i)	VEC__F(T, GetElement)(pvec, i)
#define vecGetConstElement(T, pvec, i)	VEC__F(T, GetConstElement)(pvec, i)
#define vecGetSize(T, pvec)		VEC__F(T, GetSize)(pvec)


/* interface */
#define VEC__INTERFACE(T)						\
VEC__DECL(T);								\
Vec(T)* vecNew(T);							\
void vecDelete(T, Vec(T)* vec);						\
void vecConstruct(T, Vec(T)* vec);					\
void vecDestroy(T, Vec(T)* vec);					\
void vecResize(T, Vec(T)* vec);						\
void vecPushBack(T, Vec(T)* vec, T element);				\
T* vecGetElement(T, Vec(T)* vec, uint32_t i);				\
const T* vecGetConstElement(T, const Vec(T)* vec, uint32_t i);		\
uint32_t vecGetSize(T, const Vec(T)* vec);					

