#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include "vec_.h"

inline Vec_* vec_New(uint32_t el_size) {
	Vec_* vec = malloc(sizeof(Vec_));
	if(NULL == vec)
		abort();
	vec_Construct(vec, el_size);
	return vec;
}

inline void vec_Delete(Vec_* vec) {
	vec_Destroy(vec);
	free(vec);
}

void vec_Construct(Vec_* vec, uint32_t el_size) {
	vec->el_size = el_size;
	vec->num_el = 0;
	vec->max_el = 8;
	if( NULL == (vec->end = vec->data = malloc(el_size*vec->max_el))) {
		abort();
	}
}

inline void vec_Destroy(Vec_* vec) {
	free(vec->data);
}

void vec_Resize(Vec_* vec) {
	uint32_t newsize = vec->max_el << 1;

	if( NULL == (vec->data = realloc(vec->data, vec->el_size*newsize))) {
		vec->end = NULL;
		abort();
	}
	vec->end = (uint8_t*)vec->data + vec->el_size*vec->num_el;
	vec->max_el = newsize;
}

inline void vec_PushBack(Vec_* vec, const void* element) {
	if(vec->num_el >= vec->max_el)
		vec_Resize(vec);

	memcpy(vec->end, element, vec->el_size);
	vec->end = (uint8_t*)vec->end + vec->el_size;
	vec->num_el++;
}

inline void* vec_GetElement(Vec_* vec, uint32_t i) {
	return (uint8_t*)vec->data + i*vec->el_size;
}

inline const void* vec_GetConstElement(const Vec_* vec, uint32_t i) {
	return (uint8_t*)vec->data + i*vec->el_size;
}

inline uint32_t vec_GetSize(const Vec_* vec) {
	return vec->num_el;
}

